#!/bin/bash


##  This script is distributed as part of the
##	CCsolve program.
##
##	For review only.
##
##	It may not be distributed until publication.
##
##



#!/bin/bash

#########################################################################
#
# This script prepares input files for model rebuilding using autobuild
# with the in_place option enabled
# Authors: Sebastian Rämisch, Ingemar André

echo
echo "****************************"
echo "* Start Autobuild -in_place*"
echo "****************************"
echo


#####################################
# import infos from config file

scripts=`egrep '^scripts' cc_solv.conf | awk '{print $3}'`
PDB="overall_best.pdb"
seq_file=`egrep '^seq_file' cc_solv.conf | awk '{print $3}'`
num_of_helices=`egrep '^helices_in_asu' cc_solv.conf | awk '{print $3}'`
mtz=`egrep '^mtz_file' cc_solv.conf | awk '{print $3}'`

# Prepare input files for AutoBuild in_place
# -> renumbered overall_best.pdb (overall_best.pdb_renum)

mkdir autobuild_inPlace

if [ $seq_file == "" ];then
	echo "Sequence file not defined in cc_solv.conf"
fi

cp $mtz autobuild_inPlace/
cp $seq_file autobuild_inPlace/
cp autobuild/AutoBuild_run_1_/$PDB autobuild_inPlace/

# Extract complete helices from overall_best.pdb
# The longest one, with highest sequence conservation will be superimposed onto all
# other helices in overall_best.pdb

cd autobuild_inPlace
sed -i "s/UNK/   /g" $PDB
python $scripts/renumber_pdb.py $PDB > $PDB\_renum
echo $PDB"_renum created"

echo "Extracting helices from overall_best.pdb"
echo "cmd: python $scripts/helices.py -pdb "$PDB"_renum -helices $num_of_helices"
python $scripts/helices.py -pdb $PDB"_renum" -helices $num_of_helices
sleep 5
if [ ! -f helix_1.pdb ]; then
	echo "ERROR  No helices extracted from $PDB\_renum"
	echo "Quit"
	exit 1
fi


#longest helix with highest sequence similarity:
max_len=`egrep 'ATOM\s+[0-9]+\s+N\s' helix_1.pdb | wc -l`
head -n 2 $seq_file > helices.fasta
for i in helix_?.pdb; do
	len=`pdb2fasta.py $i | sed -n 2p | awk '{ print length, $0 }' | awk '{print $1}'`
	if [ $len -eq $max_len ]; then
 		python $scripts/pdb2fasta.py $i >> helices.fasta
	fi
done
clustalw helices.fasta -gapopen=100 > clustalw.log
echo "Aligning all helix sequences to original monomer sequence:"
grep "Sequences (1:" clustalw.log | awk '{print $2 "\t" $5}' > clustal.scores
best=`sort -nr -k2 clustal.scores | head -n1 | awk '{print $1}' | cut -d: -f2 | cut -d')' -f1`
best=$[$best - 1]
best_helix="helix_$best.pdb"
echo "Best alignment: $best_helix"


# superimpose
for i in helix_?.pdb;
do
	python $scripts/superimpose.py $i $best_helix > superimpose.pdb
	echo "Superimpose $best_helix onto $i"
	# There are two helices in the pdb. We want the first one
	cut=`egrep -n 'MODEL\s+1' superimpose.pdb | cut -d: -f1`
	cut=$[$cut + 1]
	sed -n "$cut,$ p" superimpose.pdb > $i\_new
		python $scripts/renumber_pdb.py $i\_new > renum
		mv renum $i\_new
		echo "TER" >> $i\_new
	rm superimpose.pdb
done
cat helix_*new > input_autobuild_in_place.pdb

	sed -i 's/ENDMDL/TER/g' input_autobuild_in_place.pdb
	chains="A B C D E F G H I J K L M N O P Q R S T U V W X Y Z"
	counter=1
	if [ -a tmp ]; then rm tmp; fi
	cat input_autobuild_in_place.pdb | while read line
	do
		# get 1st, 2nd, 3rd ... chain from the "list"
		chain=`echo $chains | cut -d' ' -f$counter`
		if [ "$line" == "TER" ]; then
			counter=$[$counter + 1]
			echo "TER" >> tmp
			echo " chain: " $chain
			continue
		fi
		echo "$line" | sed -e "s/\(ALA\|GLY\|SER\|THR\|CYS\|LEU\|VAL\|ILE\|TYR\|TRP\|PHE\|HIS\|MET\|GLU\|GLN\|ASP\|ASN\|PRO\|LYS\|ARG\)\s[A-Z]/\1 $chain/g" >> tmp
	done
	mv tmp input_autobuild_in_place.pdb
	echo "input_autobuild_in_place.pdb successfully created"

# create monomer.seq, which has the sequence only for the longest rebuilt helix
# If the complete peptide sequence is given, it often fails to align the shorter helix
# sequence to the longer peptide sequence, which results in the wrong sequence
# to be kept.

python $scripts/make_correct_sequence.py -best_helix $best_helix
###############################
#  Run autobuild

cmd="phenix.autobuild data=$mtz model=input_autobuild_in_place.pdb seq_file=monomer.seq s_annealing=True rebuild_in_place=True min_seq_identity_percent=10 nproc=5"
echo "cmd: nice $cmd > log_rebuild &"
nice $cmd > log_rebuild &
pid=`echo $!`
printf "Check, whether autobuild is still running with:\n
ps aux | grep $pid  | grep phenix | wc -l\n\n"
echo "ps aux | grep $pid  | grep phenix | wc -l" > pid_cmd
wait $pid

cd - # protein dir
echo
echo "*******************************"
echo "* Finished Autobuild -in_place*"
echo "*******************************"
echo
Rfree=`grep "REMARK Final: r_work" autobuild_inPlace/AutoBuild_run_1_/overall_best_final_refine_001.pdb | awk '{print "Final Rfree after model rebuilding: " $8}'`
echo $Rfree

exit 0


