#!/bin/bash


##  This script is distributed as part of the
##	CCsolve program.
##
##	For review only.
##
##	It may not be distributed until publication.
##
##



#/bin/bash

#########################################################################
#
# This script prepares the input files for phaser and creates a directory
# structure. It then calls phaser to attempt phasing using the best 20
# Rosetta models.
# Authors: Ingemar André, Sebastian Rämisch

#####################################
# import infos from config file

scripts=`egrep '^scripts' cc_solv.conf | awk '{print $3}'`
rosetta_models=`egrep '^rosetta_models' cc_solv.conf | awk '{print $3}'`
rosetta_database=`egrep '^rosetta_database' cc_solv.conf | awk '{print $3}'`
num_chains=`egrep '^helices_in_complex' cc_solv.conf | awk '{print $3}'`
num_ASUchains=`egrep '^helices_in_asu' cc_solv.conf | awk '{print $3}'`
cpus=`egrep '^CPUs' cc_solv.conf | awk '{print $3}'`
mtz=`egrep '^mtz_file' cc_solv.conf | awk '{print $3}'`
chopStart=`egrep '^cut_Nterm' cc_solv.conf | awk '{print $3}'`
chopEnd=`egrep '^cut_Cterm' cc_solv.conf | awk '{print $3}'`
ros_structs=`egrep '^ros_structs' cc_solv.conf | awk '{print $3}'`

num_structs=$(($ros_structs + 1))

if command -v extract_pdbs.linuxclangrelease >/dev/null ; then
	EXTRACT_EX=extract_pdbs.linuxclangrelease
elif command -v extract_pdbs.linuxgccrelease >/dev/null; then
	EXTRACT_EX=extract_pdbs.linuxgccrelease
elif command -v extract_pdbs.static.linuxclangrelease >/dev/null; then
	EXTRACT_EX=extract_pdbs.static.linuxclangrelease
elif command -v extract_pdbs.static.linuxgccrelease >/dev/null; then
	EXTRACT_EX=extract_pdbs.static.linuxgccrelease
elif command -v extract_pdbs.linuxiccgrelease >/dev/null; then
	EXTRACT_EX=extract_pdbs.linuxiccgrelease
elif command -v extract_pdbs.static.linuxiccgrelease >/dev/null; then
	EXTRACT_EX=extract_pdbs.static.linuxiccgrelease
fi
if [ "$EXTRACT_EX" = "" ]; then
	echo "No Rosetta executable found. Exiting .."
	exit 1
else
	echo "Will use $EXTRACT_EX to extract pdb files"
fi

FIXbb_EX=`echo $EXTRACT_EX | sed -e 's/extract_pdbs/fixbb/'`
echo "Will use $FIXbb_EX to create alanine-rich chains"

echo
echo "****************"
echo "* Start Phaser *"
echo "****************"
echo
echo $$
echo " ==================================================================="
echo " The following number of residues will be cut: "
echo " N-term: $chopStart"
echo " C-term: $chopEnd"
echo " ==================================================================="

next_cmd=""
stop_at=""

case $num_chains in
1)
	stop_at="B" ;;
2)
  stop_at="C" ;;
3)
	stop_at="D" ;;
4)
	stop_at="E" ;;
5)
	stop_at="F" ;;
esac

dimer=false
doComplete=false
doPermutation=false

# if one, two or three complete complexes in ASU:
complPerASU=$(echo "scale=1; $num_ASUchains/$num_chains" | bc)
if [ "$complPerASU" = "1.0" ] || [ "$complPerASU" = "2.0" ] || [ "$complPerASU" = "3.0" ] || [ "$complPerASU" = "4.0" ] || [ "$complPerASU" = "5.0" ] || [ "$complPerASU" = "6.0" ]; then
	doComplete=true
elif [ "$num_chains" = "2" ] && [ "$num_ASUchains" = "2" ]; then
	# it's a single dimer
	dimer=true
	doPermutation=true
fi


# Check whether the files were already prepeared by a previous run
if [ ! -f phaser/preparation_completed ]; then
	# hard link the rosetta models
	mkdir phaser
	cd phaser
		echo "Tasks to run: " > tasks_to_run
		echo "Single_helix" >> tasks_to_run
		if [ $doComplete ]; then echo "complete" >> tasks_to_run
		elif [ $doPermutation ]; then echo "permutation" >> tasks_to_run
		fi
	for i in $rosetta_models/*.out; do
		cp -la $i .
	done

	pwd
	if [ -f top.out ]; then rm top.out; fi
	#extract the 20 lowest energy models
	max_score=`cat *.out | grep SCORE: | grep S_ | awk '{print $2}' | sort -n | sed -n "$num_structs"p`
	echo "Take models with score lower than: $max_score"
	perl $scripts/merge_silent.pl -p *.out -c $max_score > top.out
		$EXTRACT_EX \
		-in:file:fullatom \
		-in::file:silent_struct_type binary \
		-database $rosetta_database \
		-in:file:silent top.out

	##############################
	#  Prepare all input files

	for name in S_*.pdb; do
		dir=${name%.pdb}
		if [ -e $dir/PHASER.1.pdb ];
		then
			continue
		fi
		mkdir $dir
		cp $name $dir
		cd $dir
		mkdir single_helix
		# if one, two or three complete complexes in ASU:
		if [ "$doComplete" = "true" ]; then
			mkdir complete
		elif [ "$doPermutation" = "true" ]; then
			# it's a single dimer
			mkdir permutation
		fi
		cp ../../$mtz .
		### Asymmetric dimers from Rosett have only one chain (A)  ####
		### They need to be split into A and B first:
		if [ "$num_chains" = "2" ]; then
			len=`wc -l $name | awk '{print $1}'`
			lastA=$(($len/2))
			firstB=$(($lastA+1))
			sed -n "$firstB","$len"p $name > tmp
			sed -i "$firstB","$len"d $name
			sed -i 's/ A / B /g' tmp
			cat tmp >> $name
		fi
		for chain in A B C D E F; do
			if [ "$chain" = "$stop_at" ]; then  break; fi
			perl $scripts/getChainFromPdb.pl $name $chain > $chain"_monomer.pdb"
			python $scripts/renumber_pdb.py $chain"_monomer.pdb" > tmp
			mv tmp $chain"_monomer.pdb"
			python $scripts/pdb2fasta.py $chain"_monomer.pdb" > tmp
			tail -1 tmp > $dir"_monomer.seq"
			echo "-database $rosetta_database" > flags_ala
			echo "-in:file:s "$chain"_monomer.pdb" >> flags_ala
			echo "-resfile delta_ala.res" >> flags_ala
			python $scripts/make_res_file.py -seqfile $dir"_monomer.seq" -resfile delta_ala.res -chain $chain -res_start 1
			$FIXbb_EX @flags_ala > log_rosetta
			mv $chain"_monomer_0001.pdb" $chain"_monomer_ala.pdb"
			start=`head -1 $chain"_monomer_ala.pdb" | awk '{print $6}'`
			end=`grep "ATOM " $chain"_monomer_ala.pdb" | tail -1 | awk '{print $6}'`
			start=$((start+$chopStart))
			end=$((end-$chopEnd))
			echo grep "ATOM " $chain"_monomer_ala.pdb" | awk '{if ($6 >= '$start' && $6 <= '$end' ) print }' > $chain"_monomer_ala_chop.pdb"
			grep "ATOM " $chain"_monomer_ala.pdb" | awk '{if ($6 >= '$start' && $6 <= '$end' ) print }' > $chain"_monomer_ala_chop.pdb"
			awk '{ORS=""; for (i='$start';i<='$end';i=i+1) print substr($0,i,1)}' $dir"_monomer.seq" > $dir"_monomer_chop.seq"
			rm -f tmp
		done

		## write phaser parameters:
		printf "phaser << eof
		TITLe phaser $dir
		MODE MR_AUTO
		HKLIn $mtz
		LABIn F=FP SIGF=SIGFP
		ENSEmble single PDB A_monomer_ala_chop.pdb IDENtity 70
		COMPosition PROTein SEQuence "$dir"_monomer.seq NUMber $num_ASUchains #single
		RESOLUTION 99.0 2.1 		#rot
		XYZOUT ON			#rot
		PACK CUTOFF 5 DISTANCE 3.0	#packing 5 distance 3.0
		 SEARch ENSEmble single NUM $num_ASUchains
		 SAMPLING ROT 2			#rot
		 FINAL ROT SELECT PERCENT 75.0	#rot
		 SAVE ROT SELECT PERCENT 75.0		#rot
	#  TRANSLATE FULL			#tran
		 SAMPLING TRA 0.3			#tran
		 FINAL TRA SELECT PERCENT 75.0	#tran
		 SAVE TRA SELECT PERCENT 75.0		#tran
		 END
	eof" > phaser_run.sh
		chmod u+x phaser_run.sh
		for i in *; do
			if test -d "$i" ; then
				cp phaser_run.sh $i
				cp "$dir"_monomer.seq $i
				cp $mtz $i
			fi
		done
		rm phaser_run.sh
		cp A_monomer_ala_chop.pdb single_helix/
		cat ?_monomer_ala_chop.pdb > complete_ala_chop.pdb
		if [ -d "complete" ]; then
			cp complete_ala_chop.pdb complete/
			complPerASU=$(echo "$num_ASUchains/$num_chains" | bc) # without decimals
			sed -i "s/NUM $num_ASUchains/NUM $complPerASU/g" complete/phaser_run.sh
			sed -i "s/PDB A_monomer_ala_chop.pdb/PDB complete_ala_chop.pdb/g" complete/phaser_run.sh
		fi
		# For dimers
		if [ -d "permutation" ]; then
			cp  ?_monomer_ala_chop.pdb permutation/
			sed -i 's/single/single_A/g' permutation/phaser_run.sh
			sed -i "s/NUMber $num_ASUchains/NUMber 1/g" permutation/phaser_run.sh
			sed -i "s/NUM $num_ASUchains/NUM 1/g" permutation/phaser_run.sh
			sed -i '/ENSEmble single_A PDB/ a\  ENSEmble single_B PDB B_monomer_ala_chop.pdb IDENtity 70' permutation/phaser_run.sh
			sed -i "/COMPosition/ a\  COMPosition PROTein SEQuence "$dir"_monomer.seq NUMber 1 #single_B" permutation/phaser_run.sh
			sed -i '/SEARch ENSEmble/ a\   SEARch ENSEmble single_B NUM 1' permutation/phaser_run.sh
			sed -i '/SEARch ENSEmble single_B/ a\   PERMutations ON' permutation/phaser_run.sh
		fi
		cd ..
	done
	# Leave a message
	touch preparation_completed

else
	echo "Found preparation_completed file. Skip all preparations and continue ..."
	echo ""
	echo "************************************************************************"
	echo "  IF THERE WERE PROBLEMS AND YOU NEED TO SETUP THE FILES FOR PHASER "
	echo "  AGAIN, DELETE phaser AND START THE PROGRAM AGAIN !!!! "
	echo "************************************************************************"
	echo ""
	cd phaser
fi

##########################
#  Run PHASER


for i in single_helix complete permutation; do
	if [ "$i" = "permutation" ] && [ "$doPermutation" = "false" ]; then
		echo "No permutation runs to be done"
		break
	fi
	if [ "$i" = "complete" ] && [ "$doComplete" = "false" ]; then
		echo "No complete complex to be phased"
		break
	fi
	for name in S_*.pdb; do
		struct=${name%.pdb}
		cd $struct/$i
		jobs=`ps -U root -u root -N | grep phaser | wc -l`
		while [ $jobs -eq $cpus ]; do
			sleep 10
			jobs=`ps -U root -u root -N | grep phaser | wc -l`
		done
		# In case of a previously interrupted run, check whether phaser was run already
		completed=`tail PHASER.sum | grep '\$\$' | wc -l`
		if [ "$completed" != "1" ]; then
			echo "No sign of previous phaser attempts found."
		  echo "Starting phaser ..."
			nice ./phaser_run.sh > log_phaser &
			pid=$!
			echo "nice ./phaser_run.sh > log_phaser &" > cmd
			echo "Phaser started in $name/$i started [$pid]"
			sleep 2
		else
			pwd=`pwd`
			echo "Phaser is already done: $pwd"
		fi
		cd -
	done
done

jobs=`ps aux | grep -e "phaser$" | wc -l`
while [ $jobs -gt 0 ]; do
	sleep 60
	jobs=`ps aux | grep -e "phaser$" | wc -l`
done

echo
echo "*******************"
echo "* Phaser finished *"
echo "*******************"
echo

exit 0


