#!/bin/bash


##  This script is distributed as part of the
##	CCsolve program.
##
##	For review only.
##
##	It may not be distributed until publication.
##
##



#!/bin/bash

#########################################################################
#
# This script runs phenix.refine to identify the best models based on
# their free R value
# Authors: Ingemar André, Sebastian Rämisch

# Required: 6 available CPUs

echo
echo "**************************"
echo "* Starting Phenix refine *"
echo "**************************"
echo

#####################################
# import infos from config file

num_chains=`egrep '^helices_in_complex' cc_solv.conf | awk '{print $3}'`
num_ASUchains=`egrep '^helices_in_asu' cc_solv.conf | awk '{print $3}'`
scripts=`egrep '^scripts' cc_solv.conf | awk '{print $3}'`
mtz=`egrep '^mtz_file' cc_solv.conf | awk '{print $3}'`

################################
# CHANGE THIS IF NEEDED
#
# number of N-terminal residues to be excluded
chopStart=0
#
# number of C-terminal residues to be excluded
chopEnd=0

mkdir refine

##############################################################################################
#  Extract best PHASER.1.pdb for refinement

# 1. from single helix phasing

if [ -f tmp ]; then rm tmp; fi
for i in phaser/S_*/single_helix/PHASER.1.pdb; do
	tfz=`sed -n '3p' $i | egrep -o 'TFZ=+[0-9]+\.*[0-9]*' | tail -n 1 | egrep -o '[0-9]+\.*[0-9]*'`
	echo $tfz $i >> tmp
done
best_single=`cat tmp | sort -nr | head -n1`
best_single_strct=`cat tmp | sort -nr | head -n1 | awk '{print $2}'`
mkdir refine/single_helix
cp $best_single_strct refine/single_helix/
cp $mtz refine/single_helix/
echo "Single helix: $best_single"
echo "Single helix: $best_single" > results_phaser.txt
model=`echo $best_single_strct | cut -d/ -f2`
cp phaser/$model/A_monomer.pdb refine/single_helix/
rm tmp

# 2. from phasing the complete complex
if (( $(grep complete phaser/tasks_to_run | wc -l) == 1 )); then
space_group=`grep "Space-Group Name" phaser/S_*/complete/PHASER.sum | awk '{print $6$7}' | head -n1`
for i in phaser/S_*/complete/PHASER.1.pdb; do
	if [ "$space_group" = "P1" ]; then
		# If space group is P1, there is no TFZ score. Sort by RFZ score instead
		rfz=`sed -n '3p' $i | egrep -o 'RFZ=+[0-9]+\.*[0-9]*' | tail -n 1 | egrep -o '[0-9]+\.*[0-9]*'`
		echo $rfz $i >> tmp
	else
		tfz=`sed -n '3p' $i | egrep -o 'TFZ=+[0-9]+\.*[0-9]*' | tail -n 1 | egrep -o '[0-9]+\.*[0-9]*'`
		echo $tfz $i >> tmp
	fi
done
best_complete=`cat tmp | sort -nr | head -n1`
best_complete_strct=`cat tmp | sort -nr | head -n1 |  awk '{print $2}'`
mkdir refine/complete
cp $best_complete_strct refine/complete/
cp $mtz refine/complete/
echo "complete model: $best_complete"
echo "complete model: $best_complete" >> results_phaser.txt
model=`echo $best_complete_strct | cut -d/ -f2`
cp phaser/$model/A_monomer.pdb refine/complete/
rm tmp
else
	echo "complete model: not used for phasing. Less helices in ASU than in complex." >> results_phaser.txt
fi

# For dimers only: 3. from permutation-mode phasing
if (( $(grep permutation phaser/tasks_to_run | wc -l) == 1 )); then
	mkdir refine/permutated
	if [ -f tmp ]; then rm tmp; fi
	for i in phaser/S_*/permutation/PHASER.1.pdb; do
		tfz=`sed -n '3p' $i | egrep -o 'TFZ=+[0-9]+\.*[0-9]*' | tail -n 1 | egrep -o '[0-9]+\.*[0-9]*'`
		echo $tfz $i >> tmp
	done
	best_permut=`cat tmp | sort -nr | head -n1`
	best_permut_strct=`cat tmp | sort -nr | head -n1 |  awk '{print $2}'`
	echo "Asym dimer: $best_permut" >> results_phaser.txt
	cp $best_permut_strct refine/permutated/
	cp $mtz refine/permutated/
	model=`echo $best_permut_strct | cut -d/ -f2`
	cp phaser/$model/A_monomer.pdb phaser/$model/B_monomer.pdb refine/permutated/
	rm tmp
else
		echo "Asym dimer: not used for phasing. Protein is either not a dimer, or dimer does not have exactly two helices per ASU." >> results_phaser.txt
fi

########################
#	 Run phenix.refine

printf "\n *** Starting phenix.refine jobs ***\n"

cd refine
# Do you use other than default column labels?
labels=`egrep '^refinement' ../cc_solv.conf`
options=`egrep '^options' ../cc_solv.conf | cut -d= -f2`


for i in single_helix complete permutated; do
	if [ "$i" == "permutated" ] && (( $(grep permutation ../phaser/tasks_to_run | wc -l) == 0 )); then
		echo "Not refining permutated dimer"
		break
	fi
  if [ "$i" == "complete" ] && (( $(grep complete ../phaser/tasks_to_run | wc -l) == 0 )); then
		echo "Not refining complete ASU"
		continue
	fi

	cd $i
	echo "Start refinement: $i"
	# 1. Take PHASER.1.pdb
	cmd1="phenix.refine PHASER.1.pdb $mtz $labels $options"
	echo "$cmd1"
	echo "$cmd1" > cmd_PHASER.1
	nice $cmd1 > logRefine_PHASER.1 &
	echo "PID: $!"


	# 2.  Superimpose rosetta modelled helix onto PHASER.1.pdb
	for chain in A B C D E F; do
		perl $scripts/getChainFromPdb.pl PHASER.1.pdb $chain > $chain.pdb
		nline=`wc -l $chain.pdb | awk '{print $1}'`
		if [ "$nline" = "0"  ]; then
			rm -f $chain.pdb
		fi
	done

	rm -f starting_structure*.pdb
	for monomer in ?.pdb; do
		chain=${monomer:0:1}
		if [ "$i" = "permutation" ]; then
			python $scripts/superimpose.py $monomer $chain"_monomer.pdb" > $monomer"_sup"
		else
			python $scripts/superimpose.py $monomer A_monomer.pdb > $monomer"_sup"
		fi
		awk -v file=$monomer"_sup" '{i=0;ter=0;while (getline < file ) {if ( substr($1,1,6)=="ENDMDL") ter++;if (ter==1) print}}' $monomer"_sup" > $monomer"_sup_1"
		sed 's/'\ A\ '/'\ $chain\ '/g'  $monomer"_sup_1" > $monomer"_sup_1_chain"
		mv $monomer"_sup_1_chain" $monomer"_sup_1"
		cat $monomer"_sup_1" >> starting_structure_tmp.pdb
	done
	awk '{if ( substr($1,1,5)=="MODEL" && NR>2) print "TER"; else if (substr($1,1,5)!="MODEL") print }' starting_structure_tmp.pdb > starting_structure_tmp2.pdb
	grep -v "MODEL" starting_structure_tmp2.pdb > starting_structure_tmp3.pdb
	grep -v "ENDMDL" starting_structure_tmp3.pdb > starting_structure_tmp4.pdb
	mv starting_structure_tmp4.pdb PHASER.1_superimposed.pdb
	rm -f starting_structure_tmp* *pdb_sup*
		# Chop off ends if required
		start=`head -1 PHASER.1_superimposed.pdb | awk '{print $6}'`
		end=`grep "ATOM " PHASER.1_superimposed.pdb  | tail -1 | awk '{print $6}'`
		start=$((start+$chopStart))
		end=$((end-$chopEnd))
		echo grep "ATOM " PHASER.1_superimposed.pdb  | awk '{if ($6 >= '$start' && $6 <= '$end' ) print }' > tmp
		grep "ATOM "  PHASER.1_superimposed.pdb | awk '{if ($6 >= '$start' && $6 <= '$end' ) print }' > tmp
		mv tmp PHASER.1_superimposed.pdb
	cmd2="phenix.refine PHASER.1_superimposed.pdb $mtz $labels $options"
	echo "$cmd2"
	echo "$cmd2" > cmd_superimposed
	nice $cmd2 > logRefine_superimposed &
	echo "PID: $!"
	cd -
done
cd ..

printf "\n phenix.refine is running, now ...
waiting for all jobs to finish\n"
#grep=`ps | aux | grep`
jobs=`ps --ppid $$ | grep python | wc -l`
while [ $jobs -gt 0 ]; do
  sleep 60
	jobs=`ps --ppid $$ | grep python | wc -l`
done
sleep 30
# Get results
for i in refine/*/*refine*pdb; do grep "REMARK Final: r_work" $i | awk '{print $8 "\t" "'"$i"'"}'; done > tmp; echo "r_free  structure" > results_refine.txt; sort -k8 tmp >> results_refine.txt; cat results_refine.txt; rm tmp

echo
echo "*********************************"
echo "* run_phenix.refine.sh finished *"
echo "*********************************"
echo
